<?php

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

use WHMCS\Exception\Module\InvalidConfiguration;
use WHMCS\Domains\DomainLookup\ResultsList;
use WHMCS\Domains\DomainLookup\SearchResult;

/*
 * API URL
 */
$GLOBALS["KOUMING_API_URL"] = "https://api.kouming.com";
/*
 * 助手
 */
require_once 'lib/helper.php';
/**
 * 定义模块相关元数据
 * @return array
 */
function kouming_MetaData()
{
    return array(
        'DisplayName' => 'Kouming',
        'APIVersion' => '1.0',
    );
}

/**
 * 定义注册商配置选项。
 * @return array
 */
function kouming_getConfigArray()
{
    return [
        // 用户ID
        'user_id' => [
            'FriendlyName' => 'User ID',
            'Type' => 'text',
            'Size' => '6',
            'Default' => '10000',
            'Description' => 'Enter in User ID',
        ],
        // API密码
        'api_key' => [
            'FriendlyName' => 'API Key',
            'Type' => 'password',
            'Size' => '32',
            'Default' => '',
            'Description' => 'Enter in API Key',
        ],
    ];
}

/**
 * @param $params
 * @return void
 * 验证
 */
function kouming_config_validate($params)
{
    $public_parameter = kouming_public_parameter($params);
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"], $public_parameter)));
    if ($result["code"]) {
        throw new InvalidConfiguration($result["message"]);
    }
}

/**
 * 检查域可用性。
 */
function kouming_CheckAvailability($params)
{
    $public_parameter = kouming_public_parameter($params);
    $data = [
        "domainName" => $params["sld"] . $params["tlds"][0],
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainBasis/search", array_merge($public_parameter, $data))));
    $results = new ResultsList();
    $searchResult = new SearchResult($params['sld'], $params["tlds"][0]);
    switch ($result["code"]) {
        case "0":
            $searchResult->setStatus(SearchResult::STATUS_NOT_REGISTERED);
            break;
        default:
            $searchResult->setStatus(SearchResult::STATUS_REGISTERED);
    }
    $results->append($searchResult);
    return $results;
}

/**
 * 注册域名
 */
function kouming_RegisterDomain($params)
{
    $public_parameter = kouming_public_parameter($params);
    /*
     * 创建模板
     */
    $phone_three = $params["phonenumber"][0] . $params["phonenumber"][1] . $params["phonenumber"][2];
    $phone = "+" . $params["phonecc"] . "." . $phone_three . "-" . ltrim($params["phonenumber"], $phone_three);
    $data = [
        "category" => "0",
        "enOrganization" => $params["companyname"],
        "enLastName" => $params["lastname"],
        "enFirstName" => $params["firstname"],
        "countryCode" => $params["countrycode"],
        "enCountries" => $params["countryname"],
        "enProvince" => $params["state"],
        "enCity" => $params["city"],
        "enAddress" => $params["address1"],
        "phone" => $phone,
        "fax" => $phone,
        "zipCode" => $params["postcode"],
        "email" => $params["email"]
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainTemplate/create", array_merge($public_parameter, $data))));
    if ($result["code"]) {
        return ['error' => $result["message"]];
    }
    $template_id = $result["data"]["templateId"];
    if (!$template_id or intval($template_id) < 1) {
        return ['error' => "Contact creation failure"];
    }
    /*
     * 注册域名
     */
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "createData" => json_encode([
            [
                "domainName" => $domain,
                "years" => $params['regperiod'],
                "templateId" => $template_id,
                "dns" => $params['ns1'] . "," . $params['ns2'] . "," . $params['ns3'] . "," . $params['ns4'] . "," . $params['ns5']
            ]
        ]),
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainBasis/create", array_merge($public_parameter, $data))));
    if ($result["code"] or intval($result["data"]["failure"]) > 0) {
        //有错误时删除当前创建的模板
        @kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainTemplate/delete", array_merge($public_parameter, ["templateId" => $template_id]));
        $message = $result["message"];
        if (intval($result["data"]["failure"]) > 0) {
            $message = $result["data"]["failureMessage"][$domain];
        }
        return ['error' => $message];
    }
    return array(
        'success' => true,
    );
}

/**
 * 续费域名
 */
function kouming_RenewDomain($params)
{
    $public_parameter = kouming_public_parameter($params);
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "renewalData" => json_encode([
            [
                "domainName" => $domain,
                "years" => $params['regperiod']
            ]
        ])
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/renewal", array_merge($public_parameter, $data))));
    if ($result["code"] or intval($result["data"]["failure"]) > 0) {
        $message = $result["message"];
        if (intval($result["data"]["failure"]) > 0) {
            $message = $result["data"]["failureMessage"][$domain];
        }
        return ['error' => $message];
    }
    return array(
        'success' => true,
    );
}

/**
 * 转入域名
 */
function kouming_TransferDomain($params)
{
    return ['success' => true];
}

/**
 * 传入域传输同步。
 */
function kouming_TransferSync($params)
{
    return ['success' => true];
}

/**
 * @param $params
 * @return array|mixed
 * 获取域名信息
 */
function kouming_GetDomainInfo($params)
{
    $public_parameter = kouming_public_parameter($params);
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "domainName" => $domain
    ];
    return kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/detail", array_merge($public_parameter, $data))));
}

/**
 * 获取当前域名NS
 */
function kouming_GetNameservers($params)
{
    $result = kouming_GetDomainInfo($params);
    if (!isset($result["data"]["dns"])) {
        return ['success' => true];
    }
    $dns = explode(",", $result["data"]["dns"]);
    $dns_result = [];
    $i = 1;
    foreach ($dns as $k => $v) {
        $dns_result["ns" . $i] = $v;
        $i++;
    }
    return $dns_result;
}

/**
 * 保存NS更改。
 */
function kouming_SaveNameservers($params)
{
    $public_parameter = kouming_public_parameter($params);
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "domainName" => $domain,
        "dnsType" => 0,
        "dns" => $params['ns1'] . "," . $params['ns2'] . "," . $params['ns3'] . "," . $params['ns4'] . "," . $params['ns5'],
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/dns", array_merge($public_parameter, $data))));
    if ($result["code"]) {
        return ['error' => $result["message"]];
    }
    return array(
        'success' => true,
    );
}

/**
 * 获取当前WHOIS联系信息。
 */
function kouming_GetContactDetails($params)
{
    $public_parameter = kouming_public_parameter($params);
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "domainName" => $domain
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainTemplate/detail", array_merge($public_parameter, $data))));
    if ($result["code"]) {
        return ['error' => $result["message"]];
    }
    $data = $result["data"];
    $phone_array = explode("-", $data["phone"]);
    $fax_array = explode("-", $data["fax"]);
    return array(
        'Registrant' => array(
            'First Name' => $data["enFirstName"],
            'Last Name' => $data["enLastName"],
            'Company Name' => $data["enOrganization"],
            'Email Address' => $data["email"],
            'Address' => $data["enAddress"],
            'City' => $data["enCity"],
            'State' => $data["enProvince"],
            'Postcode' => $data["zipCode"],
            'Country' => $data["enCountries"],
            'Country Code' => $data["countryCode"],
            'Phone' => $phone_array[0] . $phone_array[1],
            'Fax' => $fax_array[0] . $fax_array[1],
        )
    );
}

/**
 * 更新给定域的WHOIS联系信息。
 */
function kouming_SaveContactDetails($params)
{
    $public_parameter = kouming_public_parameter($params);
    $contactDetails = $params['contactdetails'];
    /*
     * 创建模板
     */
    $enCountries = $contactDetails['Registrant']['Country'];
    $countryCode = $contactDetails['Registrant']['Country Code'];
    if (isset($contactDetails['Registrant']['ZIP']) and $contactDetails['Registrant']['ZIP'] != "") {
        $countryCode = $contactDetails['Registrant']['Country'];
        $enCountries = $params["countryname"];
    }
    $fax_array = explode(".", $contactDetails['Registrant']['Fax'] == "" ? $contactDetails['Registrant']['Phone'] : $contactDetails['Registrant']['Fax']);
    $phone_array = explode(".", $contactDetails['Registrant']['Phone']);
    if (count($phone_array) < 2) {
        $phone_array = $fax_array;
    }
    $fax_cc = kouming_removePunctuation($fax_array[0]);
    $fax = kouming_removePunctuation($fax_array[1]);
    $phone_cc = kouming_removePunctuation($phone_array[0]);
    $phone = kouming_removePunctuation($phone_array[1]);
    $phone_three_s = $phone[0] . $phone[1] . $phone[2];
    $phone_s = "+" . $phone_cc . "." . $phone_three_s . "-" . ltrim($phone, $phone_three_s);
    $fax_three_s = $fax[0] . $fax[1] . $fax[2];
    $fax_s = "+" . $fax_cc . "." . $fax_three_s . "-" . ltrim($fax, $fax_three_s);
    $data = [
        "category" => "0",
        "enOrganization" => $contactDetails['Registrant']['Company Name'],
        "enLastName" => $contactDetails['Registrant']['Last Name'],
        "enFirstName" => $contactDetails['Registrant']['First Name'],
        "countryCode" => $countryCode,
        "enCountries" => $enCountries,
        "enProvince" => $contactDetails['Registrant']['State'],
        "enCity" => $contactDetails['Registrant']['City'],
        "enAddress" => $contactDetails['Registrant']['Address'],
        "phone" => $phone_s,
        "fax" => $fax_s,
        "zipCode" => $contactDetails['Registrant']['Postcode'],
        "email" => $contactDetails['Registrant']['Email Address'],
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainTemplate/create", array_merge($public_parameter, $data))));
    if ($result["code"]) {
        return ['error' => $result["message"]];
    }
    $template_id = $result["data"]["templateId"];
    if (!$template_id or intval($template_id) < 1) {
        return ['error' => "Contact creation failure"];
    }
    /*
     * 过户
     */
    $domain = $params["sld"] . "." . $params['tld'];
    $data = [
        "transferData" => json_encode([
            [
                "domainName" => $domain,
                "templateId" => $template_id
            ]
        ])
    ];
    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/transfer", array_merge($public_parameter, $data))));
    if ($result["code"] or intval($result["data"]["failure"]) > 0) {
        //有错误时删除当前创建的模板
        @kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainTemplate/delete", array_merge($public_parameter, ["templateId" => $template_id]));
        $message = $result["message"];
        if (intval($result["data"]["failure"]) > 0) {
            $message = $result["data"]["failureMessage"][$domain];
        }
        return ['error' => $message];
    }
    return array(
        'success' => true,
    );
}

/**
 * 域建议设置。
 */
//function kouming_DomainSuggestionOptions()
//{
//    return ['success' => true];
//}

/**
 * 获取域建议。
 */
//function kouming_GetDomainSuggestions($params)
//{
//    return ['success' => true];
//}

/**
 * 获取注册商锁定状态。
 */
//function kouming_GetRegistrarLock($params)
//{
//    return ['success' => true];
//}

/**
 * 设置注册商锁定状态。
 */
//function kouming_SaveRegistrarLock($params)
//{
//    return ['success' => true];
//}

/**
 * 获取“DNS主机记录管理”中的DNS记录。
 */
//function kouming_GetDNS($params)
//{
//    return ['success' => true];
//}

/**
 * 更新DNS主机记录。
 */
//function kouming_SaveDNS($params)
//{
//    return ['success' => true];
//}

/**
 * 启用/禁用ID保护。
 */
//function kouming_IDProtectToggle($params)
//{
//    return ['success' => true];
//}

/**
 * 请求EEP代码。(功能完成，未开放)
 */
//function kouming_GetEPPCode($params)
//{
//    $public_parameter = kouming_public_parameter($params);
//    $domain = $params["sld"] . "." . $params['tld'];
//    $data = [
//        "transferData" => json_encode([
//            [
//                "domainName" => $domain,
//                "reason" => "Other reasons",
//                "sendEmail" => "1",
//            ]
//        ])
//    ];
//    $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/transferOut", array_merge($public_parameter, $data))));
//    if ($result["code"] or intval($result["data"]["failure"]) > 0) {
//        $message = $result["message"];
//        if (intval($result["data"]["failure"]) > 0) {
//            $message = $result["data"]["failureMessage"][$domain];
//            //如果申请过就去查转移密码
//            if (strpos($message, 'domain name Transfer Out progress') !== false) {
//                $message_array = explode(":", $message);
//                $queueId = $message_array[1];
//                $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/transferOutStatus", array_merge($public_parameter, ["queueId" => $queueId]))));
//                if ($result["code"]) {
//                    return ['success' => true];
//                }
//                return ['eppcode' => $result["data"]["password"]];
//            }
//        }
//        return ['error' => $message];
//    }
//    return array(
//        'success' => true,
//    );
//}

/**
 * 释放域。
 */
//function kouming_ReleaseDomain($params)
//{
//    return ['success' => true];
//}

/**
 * 删除域。
 */
//function kouming_RequestDelete($params)
//{
//    return ['success' => true];
//}

/**
 * 注册一个名称服务器。
 */
//function kouming_RegisterNameserver($params)
//{
//    return ['success' => true];
//}

/**
 * 修改名称服务器。
 */
//function kouming_ModifyNameserver($params)
//{
//    return ['success' => true];
//}

/**
 * 删除名称服务器。
 */
//function kouming_DeleteNameserver($params)
//{
//    return ['success' => true];
//}

/**
 * 同步域状态和过期日期。
 */
//function kouming_Sync($params)
//{
//    return ['success' => true];
//}


/**
 * 客户区自定义按钮数组。
 */
//function kouming_ClientAreaCustomButtonArray()
//{
//    return ['success' => true];
//}

/**
 * 客户端区域允许的功能。
 */
//function kouming_ClientAreaAllowedFunctions()
//{
//    return ['success' => true];
//}

/**
 * 自定义模块功能:Push
 */
//function kouming_push($params)
//{
//    return ['success' => true];
//}

/**
 * 客户端区域输出。
 */
function kouming_ClientArea($params)
{
    $result = kouming_GetDomainInfo($params);
    $method = $_GET["kouming_method"];
    if (isset($_GET["kouming_method"])) {
        $public_parameter = kouming_public_parameter($params);
        if ($method == "login") {
            $domain = $params["sld"] . "." . $params['tld'];
            $data = [
                "domainName" => $domain,
                "password" => $result["data"]["password"]
            ];
            $result = kouming_objectArray(json_decode(kouming_post($GLOBALS["KOUMING_API_URL"] . "/domainManagement/consoleLogin", array_merge($public_parameter, $data))));
            if ($result["code"]) {
                return
                    '<br/>
<div class="alert alert-danger">
    ' . $result["message"] . '
</div>';
            }
            header("Location: " . $result["data"]["url"]);
            exit();
        }
    }
    if (!isset($result["data"]["domain"])) {
        return '<br/>
<div class="alert alert-danger">
    Please contact the administrator
</div>';
    }
    return '<br/>
<div class="alert alert-success">Control Panel : 
    <a href="clientarea.php?action=domaindetails&id=' . $params["domainid"] . '&kouming_method=login" target="_blank" class="button btn btn-default">Click to Login</a>
</div>';
}